/// <reference types="chart.js" />
import { AbstractOutputProps } from './abstract-output-component';
import { AbstractTreeOutputComponent, AbstractTreeOutputState } from './abstract-tree-output-component';
import { ResponseStatus } from 'tsp-typescript-client/lib/models/response/responses';
import { Entry } from 'tsp-typescript-client/lib/models/entry';
import ColumnHeader from './utils/filter-tree/column-header';
import { TreeNode } from './utils/filter-tree/tree-node';
import { XyEntry } from 'tsp-typescript-client/lib/models/xy';
import * as React from 'react';
import { TimeRange } from 'traceviewer-base/lib/utils/time-range';
export declare const ZOOM_IN_RATE = 0.8;
export declare const ZOOM_OUT_RATE = 1.25;
export declare const FLAG_ZOOM_IN = true;
export declare const FLAG_ZOOM_OUT = false;
export declare const FLAG_PAN_LEFT = true;
export declare const FLAG_PAN_RIGHT = false;
export declare enum XY_OUTPUT_KEY_ACTIONS {
    ZOOM_IN = 0,
    ZOOM_OUT = 1,
    PAN_LEFT = 2,
    PAN_RIGHT = 3,
    SHIFT_PRESS = 4
}
export declare enum MouseButton {
    NONE = -1,
    LEFT = 0,
    MID = 1,
    RIGHT = 2
}
export declare type AbstractXYOutputState = AbstractTreeOutputState & {
    selectedSeriesId: number[];
    xyTree: Entry[];
    checkedSeries: number[];
    collapsedNodes: number[];
    orderedNodes: number[];
    xyData: any;
    columns: ColumnHeader[];
    allMax: number;
    allMin: number;
    cursor?: string;
};
export declare abstract class AbstractXYOutputComponent<P extends AbstractOutputProps, S extends AbstractXYOutputState> extends AbstractTreeOutputComponent<P, S> {
    private yAxisRef;
    protected divRef: any;
    protected chartRef: any;
    private currentColorIndex;
    private colorMap;
    protected margin: {
        top: number;
        right: number;
        bottom: number;
        left: number;
    };
    protected isScatterPlot: boolean;
    protected isBarPlot: boolean;
    protected keyActionMap: Map<string, XY_OUTPUT_KEY_ACTIONS>;
    private preventDefaultHandler;
    protected posPixelSelect: number;
    protected positionYMove: number;
    protected isMouseLeave: boolean;
    protected clickedMouseButton: MouseButton;
    protected mouseIsDown: boolean;
    protected isPanning: boolean;
    protected isSelecting: boolean;
    protected positionXMove: number;
    protected startPositionMouseRightClick: bigint;
    protected endSelection: (event: MouseEvent) => void;
    private plugin;
    private _debouncedUpdateXY;
    constructor(props: P);
    protected abstract afterChartDraw(ctx: CanvasRenderingContext2D | null, chartArea?: Chart.ChartArea | null): void;
    /**
     * Get the display range for the current chart.
     * For generic XY output, the displayed range is the view range.
     * For the overview output, the displayed range is the trace full range.
     */
    protected abstract getDisplayedRange(): TimeRange;
    /**
     * Returns the time that the chart should zooms into/out to.
     */
    protected abstract getZoomTime(): bigint;
    protected onToggleCollapse(id: number, nodes: TreeNode[]): void;
    protected onOrderChange(ids: number[]): void;
    protected onToggleCheck(ids: number[]): void;
    private updateRange;
    private applySelectionZoom;
    protected updateSelection(): void;
    componentDidMount(): void;
    componentDidUpdate(prevProps: AbstractOutputProps, prevState: AbstractXYOutputState): void;
    componentWillUnmount(): void;
    fetchTree(): Promise<ResponseStatus>;
    getAllCheckedIds(entries: Array<XyEntry>): Array<number>;
    renderTree(): React.ReactNode | undefined;
    renderYAxis(): React.ReactNode;
    resultsAreEmpty(): boolean;
    setFocus(): void;
    protected chooseChart(): JSX.Element;
    private updateXY;
    private viewSpinner;
    private buildScatterData;
    private buildXYData;
    private getSeriesColor;
    private calculateYRange;
    protected getTimeForX(x: number): bigint;
    protected getXForTime(time: bigint): number;
    protected zoom(isZoomIn: boolean): void;
    protected pan(panLeft: boolean): void;
    protected tooltip(x: number, y: number): void;
    protected hideTooltip(): void;
}
//# sourceMappingURL=abstract-xy-output-component.d.ts.map